import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EcoreUtil;

/**
 * The {@link EcoreUtil.Copier} is very restrictive with it's
 * intended-to-be-overridden API. It is not allowed to return "null" to mark an
 * object or reference as not to be copied. In order to copy only parts of an
 * object tree using the {@link NullAwareCopier} - just return "null" in a
 * {@link EcoreUtil.Copier#getTarget(org.eclipse.emf.ecore.EClass)} or
 * {@link EcoreUtil.Copier#getTarget(EStructuralFeature)} call.
 * 
 * @author Enrico Schnepel
 */
public abstract class NullAwareCopier extends EcoreUtil.Copier {
	@Override
	public EObject copy(EObject eObject) {
		EObject copyEObject = get(eObject);
		if (null == copyEObject) {
			if (null == getTarget(eObject.eClass())) {
				put(eObject, null);
			}
			copyEObject = super.copy(eObject);
		}
		return copyEObject;
	}

	@Override
	protected EObject createCopy(EObject eObject) {
		if (null == getTarget(eObject.eClass()))
			return null;
		else
			return super.createCopy(eObject);
	}

	@Override
	protected void copyContainment(EReference eReference, EObject eObject,
			EObject copyEObject) {
		if (eObject.eIsSet(eReference)) {
			EStructuralFeature targetClass = getTarget(eReference);
			if (null != targetClass)
				super.copyContainment(eReference, eObject, copyEObject);
		}
	}

	@Override
	protected void copyReference(EReference eReference, EObject eObject,
			EObject copyEObject) {
		if (eObject.eIsSet(eReference)) {
			EStructuralFeature targetClass = getTarget(eReference);
			if (null != targetClass)
				super.copyReference(eReference, eObject, copyEObject);
		}
	}
}